% This script can be used to produce figure 3 & 4 in DDPCA paper. Note the
% running time is around 40 min.
clear
clc;
rng('default')
t1 = tic;
%% Some Meta Parameter
normalization = false;
num_fold = 5;

screening_size = 1000;

K_list = [1,2,3];
feature_list = 6:screening_size;

data_source = 'lung';

%% Load Data
if (strcmp(data_source,'lung'))
    Data = load('Data/lungcancer.txt'); 
    Class = load('Data/lungcancery.txt');
    numselect_default = 251; 
elseif (strcmp(data_source,'breast'))
    Data = load('Data/WangBreast2005.txt'); 
    Class = load('Data/WangBreast2005.y.txt');
    numselect_default = 728;
end

%% Data Preprocessing
[p,n] = size(Data);

% normalization
if normalization
    gm = mean(Data,2); 
    gsd = std(Data,0,2);
    Data = (Data - repmat(gm, 1, n))./repmat(gsd, 1, n);
end

%% main loop
Y = Class;

index_0 = find(Y'==0);
index_1 = find(Y'==1);
l0 = length(index_0);
l1 = length(index_1);

% stratified sampling 
randindex_0 = index_0(randperm(l0));
randindex_1 = index_1(randperm(l1));

cutpoint_0 = 1:fix(l0/num_fold):l0;
cutpoint_1 = 1:fix(l1/num_fold):l1;

K_len = length(K_list);
feature_len = length(feature_list);


error_poet = zeros(feature_len,num_fold,K_len);
error_DD = zeros(feature_len,num_fold,K_len);
error_inde = zeros(feature_len,num_fold,K_len);

for i=1:num_fold
    % split training and testing data
    if (i<num_fold)
        test_index = [randindex_0(cutpoint_0(i):(cutpoint_0(i+1)-1)),randindex_1(cutpoint_1(i):(cutpoint_1(i+1)-1))];
    else
        test_index = [randindex_0(cutpoint_0(i):end),randindex_1(cutpoint_1(i):end)];
    end
    
    train_index = setdiff(1:n,test_index);
    
    Data1 = Data(:,train_index);
    Class1 = Class(train_index);
    
    % screening
    n1 = sum(Class1==1);
    n0 = sum(Class1==0);
    mu_plus = mean(Data1(:,Class1==1),2);
    mu_minus = mean(Data1(:,Class1==0),2);
    s_plus = std(Data1(:,Class1==1),0,2);
    s_minus = std(Data1(:,Class1==0),0,2);
    s_pool = sqrt(((n1-1)*s_plus.^2 + (n0-1)*s_minus.^2)/(n1+n0-2));
    nn = sqrt(1/n1 + 1/n0);
    t_score = (mu_plus - mu_minus)./(nn*s_pool);
    [~,sorted_index] = sort(abs(t_score),'descend');
    post_screening_index = sorted_index(1:screening_size);
    
    Data_sub = Data1(post_screening_index,:);
    t_score_sub = t_score(post_screening_index);
    mu_plus_sub = mu_plus(post_screening_index);
    mu_minus_sub = mu_minus(post_screening_index);
    s_pool_sub = s_pool(post_screening_index);
    
    % test data and its normalization
    test_data = Data(post_screening_index,test_index);
    test_class = Class(test_index);
    
    test_data_normalized = test_data - repmat((mu_plus_sub + mu_minus_sub)/2,1,length(test_class));
    test_data_normalized = test_data_normalized./repmat(s_pool_sub,1,length(test_class));
    
    
    % estimate precision matrix
    
    for j=1:K_len
        Sigma_sample = cov(Data_sub');
        
        % Independence rule
        SigmaY_inde = diag(diag(Sigma_sample));
        
        % POET
        K_current = K_list(j);
        
        C_min = Cmin(Data_sub,K_current,'hard');
        if isnan(C_min)
            C_min = 0;
        end
        C_best = Cross_val(Data_sub,K_current,'hard',C_min);
        [SigmaY_poet,~] = POET(Data_sub,K_current,C_best,'hard');
        
        % DD-POET
        Sigma_sample = cov(Data_sub');
        [ T_DD,rem_DD ] = LRDD(Sigma_sample,K_current,1);
        rem_DD=ProjSDDplus(rem_DD);
        SigmaY_DD = rem_DD + T_DD;
        
        
        tilde_Z_poet = SigmaY_poet\t_score_sub;
        tilde_Z_DD = SigmaY_DD\t_score_sub;
        tilde_Z_inde = SigmaY_inde\t_score_sub;
        
        [~,poet_index] = sort(abs(tilde_Z_poet),'descend');
        [~,DD_index] = sort(abs(tilde_Z_DD),'descend');
        [~,inde_index] = sort(abs(tilde_Z_inde),'descend');
        
        sign_Z_poet = sign(tilde_Z_poet);
        sign_Z_DD = sign(tilde_Z_DD);
        sign_Z_inde = sign(tilde_Z_inde);
        
        unweighted_poet = SigmaY_poet\test_data_normalized;
        unweighted_DD = SigmaY_DD\test_data_normalized;
        unweighted_inde = SigmaY_inde\test_data_normalized;
        
        for w=1:feature_len
            weight_poet = zeros(screening_size,1);
            weight_poet(poet_index(1:w)) = sign_Z_poet(poet_index(1:w));
            
            weight_DD = zeros(screening_size,1);
            weight_DD(DD_index(1:w)) = sign_Z_DD(DD_index(1:w));
            
            weight_inde = zeros(screening_size,1);
            weight_inde(inde_index(1:w)) = sign_Z_inde(inde_index(1:w));
            
            % Error on test data
            
            
            error_poet(w,i,j) = sum(xor((weight_poet'*unweighted_poet)>=0,test_class'));
            error_DD(w,i,j) = sum(xor((weight_DD'*unweighted_DD)>=0,test_class'));
            error_inde(w,i,j) = sum(xor((weight_inde'*unweighted_inde)>=0,test_class'));
            
        end
        
    end
    
end
%% compare performance
msr_poet = sum(error_poet,2);
msr_DD = sum(error_DD,2);
msr_inde = sum(error_inde,2);


%% figures

if (strcmp(data_source,'lung'))
    max_feature_in_plot = 100;
    plot_step = 5;
else
    max_feature_in_plot = 700;
    plot_step = 20;
end

inital_num = feature_list(1);
plot_feature_list = inital_num:plot_step:max_feature_in_plot;
linewidth = 4;

figure

subplot(1,3,1)
hold on 
xlim([0 max_feature_in_plot])
plot(plot_feature_list,msr_poet(plot_feature_list-inital_num + 1,1,1),':b','LineWidth',linewidth)
plot(plot_feature_list,msr_DD(plot_feature_list-inital_num + 1,1,1),'-r','LineWidth',linewidth)
plot(plot_feature_list,msr_inde(plot_feature_list-inital_num + 1,1,1),'-.k','LineWidth',linewidth)
xlabel('Number of Features after Screening','FontSize',14)
set(gca, 'FontSize', 14)
title(['K = ', num2str(K_list(1))],'FontSize',16)
legend({'POET','DD-PCA','FAIR'},'FontSize',15)
hold off

subplot(1,3,2)
hold on 
xlim([0 max_feature_in_plot])
plot(plot_feature_list,msr_poet(plot_feature_list-inital_num + 1,1,2),':b','LineWidth',linewidth)
plot(plot_feature_list,msr_DD(plot_feature_list-inital_num + 1,1,2),'-r','LineWidth',linewidth)
plot(plot_feature_list,msr_inde(plot_feature_list-inital_num + 1,1,2),'-.k','LineWidth',linewidth)
xlabel('Number of Features after Screening','FontSize',14)
set(gca, 'FontSize', 14)
title(['K = ', num2str(K_list(2))],'FontSize',16)
legend({'POET','DD-PCA','FAIR'},'FontSize',15)
hold off

subplot(1,3,3)
hold on 
xlim([0 max_feature_in_plot])
plot(plot_feature_list,msr_poet(plot_feature_list-inital_num + 1,1,3),':b','LineWidth',linewidth)
plot(plot_feature_list,msr_DD(plot_feature_list-inital_num + 1,1,3),'-r','LineWidth',linewidth)
plot(plot_feature_list,msr_inde(plot_feature_list-inital_num + 1,1,3),'-.k','LineWidth',linewidth)
xlabel('Number of Features after Screening','FontSize',14)
set(gca, 'FontSize', 14)
title(['K = ', num2str(K_list(3))],'FontSize',16)
legend({'POET','DD-PCA','FAIR'},'FontSize',15)
hold off


%%
fig = gcf;
fig.PaperUnits = 'inches';
fig.PaperPosition = [0 0 16 9];
%fig.PaperPositionMode = 'auto';
fig.PaperSize = [16,9];
print(['LDA_', data_source, '_',num2str(screening_size)],'-dpdf','-r250')
%%
save(['LDA_', data_source, '_',num2str(screening_size),'.mat'])
toc(t1)